!SLIDE

# Problem

!SLIDE bullets incremental

## Scalable Archival Storage ##

* Social Media
* Web Pages
* S3 Replacement

!SLIDE center

<a href="http://basho.com"><img src="riak-logo.png"/></a>

!SLIDE bullets incremental

# Riak

* Distributed
* Fault-Tolerant
* Scales Up and Down
* Key-Value Access
* Content-Agnostic

!SLIDE bullets incremental

## Archive Tweets ##

* All of them!!!

!SLIDE smallest

## Twitter Streaming API ##

    @@@ ruby
    require 'tweetstream'

    TweetStream::Client.new('user', 'password').track('justin bieber') do |status|
      puts "#{status.text}"
    end

!SLIDE smaller

## Store them into Riak

    @@@ ruby
    require 'riak'

    riak = Riak::Client.new
    bucket = riak.bucket('tweets')
    tweet = Riak::RObject.new(bucket, "key")
    tweet.content_type = "application/json"
    tweet.data = "{'text': 'tweet!'}"
    tweet.store

!SLIDE smallest

## Under the covers - Storing Data

    > PUT /riak/tweets/key HTTP/1.1
    > User-Agent: curl/7.19.7 (universal-apple-darwin10.0)
    > Host: localhost:8098
    > Accept: */*
    > Content-Type: application/json
    > Content-Length: 18

!SLIDE smallest

## Under the covers

    < HTTP/1.1 200 OK
    < X-Riak-Vclock: a85hYGBgzmDKBVIsTDb6JzOYEhnzWBmit8gf=
    < Vary: Accept-Encoding
    < Server: MochiWeb/1.1 WebMachine/1.7.2 (participate in the frantic)
    < Link: </riak/tweets>; rel="up"
    < Date: Tue, 05 Oct 2010 14:38:40 GMT
    < Content-Type: application/json
    < Content-Length: 18

!SLIDE smallest

## Under the covers - Fetching Data

    > GET /riak/tweets/key HTTP/1.1
    > User-Agent: curl/7.19.7 (universal-apple-darwin10.0)
    > Host: localhost:8098
    > Accept: */*

!SLIDE smallest

## Fetching Data

    < HTTP/1.1 200 OK
    < X-Riak-Vclock: a85hYGBgzmDKBVIsTDb6JzOYEhnzWBmit8gf=
    < Vary: Accept-Encoding
    < Server: MochiWeb/1.1 WebMachine/1.7.2 (participate in the frantic)
    < Link: </riak/tweets>; rel="up"
    < Last-Modified: Tue, 05 Oct 2010 14:38:40 GMT
    < ETag: 71hTllswGrWbI2O3WEkZYE
    < Date: Tue, 05 Oct 2010 14:40:04 GMT
    < Content-Type: application/json
    < Content-Length: 18
    < 
    {'text': 'tweet!'}

!SLIDE smallest

# Tweets -> Riak


    @@@ ruby
    require 'tweetstream'
    require 'riak'

    riak = Riak::Client.new
    TweetStream::Client.new('user', 'password').track('justin bieber') do |tweet|
      bucket = riak.bucket('tweets')
      tweet_object = Riak::RObject.new(bucket, tweet.id_str)
      tweet_object.content_type = "application/json"
      tweet_object.data = tweet
      tweet_object.store
    end


!SLIDE bullets incremental

# Analyze the data

* MapReduce
* JavaScript or Erlang

!SLIDE smaller

# Map Function

    @@@ javascript
    function(value) {
      var doc = Riak.mapValuesJson(value)[0];
      if (doc.text.match(/hate/)) {
        return [1];
      } else {
        return [0];
      }
    }

!SLIDE smaller

# Reduce Function

    @@@ javascript
    function(values) {
      if (values.length > 0) {
        return [values.reduce(function(previous, current) {
          return previous + current;
        })];
      }
      else {
        return [0];
      }
    }

!SLIDE smaller

    @@@ ruby
    results = Riak::MapReduce.new(riak).
                add("tweets").
                map(mapFunction).
                reduce(reduceFunction).run

!SLIDE smaller

## Connect Tweets with Links

    @@@ ruby
    tweet.links.add Riak::Link.new('tweets', '12345', 'in_reply_to') 
    tweet.store

!SLIDE smallest

## Under the Covers

    > PUT /riak/tweets/key HTTP/1.1
    > Link: </riak/tweets/12345>; riaktag="in_reply_to"

!SLIDE smaller

## Walk the Graph of Links

    @@@ ruby
    tweet.walk(bucket: "_", tag: "in_reply_to")

!SLIDE bullets incremental

## Full Text Search

* Riak Search
* Like Solr for your Riak

!SLIDE small

    @@@ ruby
    require 'riak/search'

    client = Riak::Client.new :solr => "/solr"
    client['tweets'].enable_index!
    client.search('tweets', 'hate')

!SLIDE small

## Search ♥ MapReduce

    @@@ ruby
    Riak::MapReduce.new(client).
      search("tweets", "text:love").
      map("Riak.mapValuesJson", :keep => true).
      run

!SLIDE bullets incremental

# Why Riak?

* Oblivious to data and failures
* Increase capacity on demand
* MapReduce and Search for analysis
